@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause
 /*
- * Copyright 2019-2025 Amazon.com, Inc. or its affiliates. All rights reserved.
+ * Copyright 2019-2026 Amazon.com, Inc. or its affiliates. All rights reserved.
  */
 
 #include <assert.h>
@@ -549,6 +549,7 @@ static enum ibv_wc_status to_ibv_status(enum efa_io_comp_status status)
 		return IBV_WC_REM_INV_RD_REQ_ERR;
 	case EFA_IO_COMP_STATUS_REMOTE_ERROR_BAD_STATUS:
 		return IBV_WC_BAD_RESP_ERR;
+	case EFA_IO_COMP_STATUS_REMOTE_ERROR_FEATURE_MISMATCH:
 	case EFA_IO_COMP_STATUS_REMOTE_ERROR_BAD_LENGTH:
 		return IBV_WC_REM_INV_REQ_ERR;
 	case EFA_IO_COMP_STATUS_LOCAL_ERROR_UNRESP_REMOTE:
@@ -816,7 +817,7 @@ static inline int efa_poll_sub_cq(struct efa_cq *cq, struct efa_sub_cq *sub_cq,
 		 * from the table.
 		 */
 		*cur_qp = ctx->qp_table[qpn & ctx->qp_table_sz_m1];
-		if (!*cur_qp) {
+		if (!*cur_qp || qpn != (*cur_qp)->verbs_qp.qp.qp_num) {
 			cq->cur_wq = NULL;
 			verbs_err(&ctx->ibvctx,
 				  "QP[%u] does not exist in QP table\n",
@@ -1057,7 +1058,8 @@ static void efa_cq_fill_pfns(struct efa_cq *cq,
 	if (cq->num_sub_cqs == 1)
 		cq_pfns_mask |= SINGLE_SUB_CQ_PFNS;
 
-	if (attr->flags & IBV_CREATE_CQ_ATTR_SINGLE_THREADED)
+	if ((cq->parent_domain && cq->parent_domain->td) ||
+	    attr->flags & IBV_CREATE_CQ_ATTR_SINGLE_THREADED)
 		cq_pfns_mask |= SINGLE_THREAD_PFNS;
 
 	cq_ops = &base_ops[cq_pfns_mask];
@@ -1285,7 +1287,7 @@ struct ibv_cq_ex *efadv_create_cq(struct ibv_context *ibvctx,
 
 	if (!vext_field_avail(struct efadv_cq_init_attr, wc_flags, inlen) ||
 	    efa_attr->comp_mask ||
-	    (inlen > sizeof(efa_attr) && !is_ext_cleared(efa_attr, inlen))) {
+	    (inlen > sizeof(*efa_attr) && !is_ext_cleared(efa_attr, inlen))) {
 		verbs_err(verbs_get_ctx(ibvctx), "Compatibility issues\n");
 		errno = EINVAL;
 		return NULL;
@@ -2170,7 +2172,7 @@ static void efa_set_common_ctrl_flags(struct efa_io_tx_meta_desc *desc,
 	EFA_SET(&desc->ctrl2, EFA_IO_TX_META_DESC_COMP_REQ, 1);
 }
 
-#ifdef LTTNG_ENABLED
+#if defined(LTTNG_ENABLED) || defined(USDT_ENABLED)
 static uint32_t efa_get_wqe_length(struct efa_io_tx_wqe *tx_wqe)
 {
 	enum efa_io_send_op_type op_type;
@@ -2290,6 +2292,8 @@ int efa_post_send(struct ibv_qp *ibvqp, struct ibv_send_wr *wr,
 
 	if (wq->need_lock)
 		mmio_wc_spinlock(&wq->wqlock);
+	else
+		mmio_wc_start();
 
 	while (wr) {
 		err = efa_post_send_validate_wr(qp, wr);
@@ -2636,6 +2640,8 @@ static void efa_send_wr_start(struct ibv_qp_ex *ibvqpx)
 
 	if (qp->sq.wq.need_lock)
 		mmio_wc_spinlock(&qp->sq.wq.wqlock);
+	else
+		mmio_wc_start();
 
 	qp->wr_session_err = 0;
 	sq->num_wqe_pending = 0;
